@@ -28,9 +28,9 @@
 **
 ** The procedure call above will create and register a new VFS shim named
 ** "multiplex".  The multiplex VFS will use the VFS named by zOrigVfsName to
-** do the actual disk I/O.  (The zOrigVfsName parameter may be NULL, in 
+** do the actual disk I/O.  (The zOrigVfsName parameter may be NULL, in
 ** which case the default VFS at the moment sqlite3_multiplex_initialize()
-** is called will be used as the underlying real VFS.)  
+** is called will be used as the underlying real VFS.)
 **
 ** If the makeDefault parameter is TRUE then multiplex becomes the new
 ** default VFS.  Otherwise, you can use the multiplex VFS by specifying
@@ -39,7 +39,7 @@
 ** URI.
 **
 ** The multiplex VFS allows databases up to 32 GiB in size.  But it splits
-** the files up into smaller pieces, so that they will work even on 
+** the files up into smaller pieces, so that they will work even on
 ** filesystems that do not support large files.  The default chunk size
 ** is 2147418112 bytes (which is 64KiB less than 2GiB) but this can be
 ** changed at compile-time by defining the SQLITE_MULTIPLEX_CHUNK_SIZE
@@ -58,29 +58,16 @@
 #endif
 #include "sqlite3ext.h"
 
-/* 
-** These should be defined to be the same as the values in 
+/*
+** These should be defined to be the same as the values in
 ** sqliteInt.h.  They are defined separately here so that
-** the multiplex VFS shim can be built as a loadable 
+** the multiplex VFS shim can be built as a loadable
 ** module.
 */
 #define UNUSED_PARAMETER(x) (void)(x)
 #define MAX_PAGE_SIZE       0x10000
 #define DEFAULT_SECTOR_SIZE 0x1000
 
-/*
-** For a build without mutexes, no-op the mutex calls.
-*/
-#if defined(SQLITE_THREADSAFE) && SQLITE_THREADSAFE==0
-#define sqlite3_mutex_alloc(X)    ((sqlite3_mutex*)8)
-#define sqlite3_mutex_free(X)
-#define sqlite3_mutex_enter(X)
-#define sqlite3_mutex_try(X)      SQLITE_OK
-#define sqlite3_mutex_leave(X)
-#define sqlite3_mutex_held(X)     ((void)(X),1)
-#define sqlite3_mutex_notheld(X)  ((void)(X),1)
-#endif /* SQLITE_THREADSAFE==0 */
-
 /* Maximum chunk number */
 #define MX_CHUNK_NUMBER 299
 
@@ -96,7 +83,7 @@
 #endif
 
 /* This is the limit on the chunk size.  It may be changed by calling
-** the xFileControl() interface.  It will be rounded up to a 
+** the xFileControl() interface.  It will be rounded up to a
 ** multiple of MAX_PAGE_SIZE.  We default it here to 2GiB less 64KiB.
 */
 #ifndef SQLITE_MULTIPLEX_CHUNK_SIZE
@@ -139,12 +126,11 @@ struct multiplexGroup {
   unsigned int szChunk;            /* Chunk size used for this group */
   unsigned char bEnabled;          /* TRUE to use Multiplex VFS for this file */
   unsigned char bTruncate;         /* TRUE to enable truncation of databases */
-  multiplexGroup *pNext, *pPrev;   /* Doubly linked list of all group objects */
 };
 
 /*
 ** An instance of the following object represents each open connection
-** to a file that is multiplex'ed.  This object is a 
+** to a file that is multiplex'ed.  This object is a
 ** subclass of sqlite3_file.  The sqlite3_file object for the underlying
 ** VFS is appended to this structure.
 */
@@ -171,11 +157,11 @@ static struct {
   */
   sqlite3_vfs sThisVfs;
 
-  /* The sIoMethods defines the methods used by sqlite3_file objects 
+  /* The sIoMethods defines the methods used by sqlite3_file objects
   ** associated with this shim.  It is initialized at start-time and does
   ** not require a mutex.
   **
-  ** When the underlying VFS is called to open a file, it might return 
+  ** When the underlying VFS is called to open a file, it might return
   ** either a version 1 or a version 2 sqlite3_file object.  This shim
   ** has to create a wrapper sqlite3_file of the same version.  Hence
   ** there are two I/O method structures, one for version 1 and the other
@@ -187,25 +173,9 @@ static struct {
   /* True when this shim has been initialized.
   */
   int isInitialized;
-
-  /* For run-time access any of the other global data structures in this
-  ** shim, the following mutex must be held.
-  */
-  sqlite3_mutex *pMutex;
-
-  /* List of multiplexGroup objects.
-  */
-  multiplexGroup *pGroups;
 } gMultiplex;
 
 /************************* Utility Routines *********************************/
-/*
-** Acquire and release the mutex used to serialize access to the
-** list of multiplexGroups.
-*/
-static void multiplexEnter(void){ sqlite3_mutex_enter(gMultiplex.pMutex); }
-static void multiplexLeave(void){ sqlite3_mutex_leave(gMultiplex.pMutex); }
-
 /*
 ** Compute a string length that is limited to what can be stored in
 ** lower 30 bits of a 32-bit signed integer.
@@ -229,16 +199,16 @@ static int multiplexStrlen30(const char *z){
 ** nul-terminator.
 **
 ** If iChunk is 0 (or 400 - the number for the first journal file chunk),
-** the output is a copy of the input string. Otherwise, if 
+** the output is a copy of the input string. Otherwise, if
 ** SQLITE_ENABLE_8_3_NAMES is not defined or the input buffer does not contain
-** a "." character, then the output is a copy of the input string with the 
-** three-digit zero-padded decimal representation if iChunk appended to it. 
+** a "." character, then the output is a copy of the input string with the
+** three-digit zero-padded decimal representation if iChunk appended to it.
 ** For example:
 **
 **   zBase="test.db", iChunk=4  ->  zOut="test.db004"
 **
 ** Or, if SQLITE_ENABLE_8_3_NAMES is defined and the input buffer contains
-** a "." character, then everything after the "." is replaced by the 
+** a "." character, then everything after the "." is replaced by the
 ** three-digit representation of iChunk.
 **
 **   zBase="test.db", iChunk=4  ->  zOut="test.004"
@@ -262,12 +232,12 @@ static void multiplexFilename(
     if( i>=n-4 ) n = i+1;
     if( flags & SQLITE_OPEN_MAIN_JOURNAL ){
       /* The extensions on overflow files for main databases are 001, 002,
-      ** 003 and so forth.  To avoid name collisions, add 400 to the 
+      ** 003 and so forth.  To avoid name collisions, add 400 to the
       ** extensions of journal files so that they are 401, 402, 403, ....
       */
       iChunk += SQLITE_MULTIPLEX_JOURNAL_8_3_OFFSET;
     }else if( flags & SQLITE_OPEN_WAL ){
-      /* To avoid name collisions, add 700 to the 
+      /* To avoid name collisions, add 700 to the
       ** extensions of WAL files so that they are 701, 702, 703, ....
       */
       iChunk += SQLITE_MULTIPLEX_WAL_8_3_OFFSET;
@@ -286,7 +256,7 @@ static void multiplexFilename(
 static int multiplexSubFilename(multiplexGroup *pGroup, int iChunk){
   if( iChunk>=pGroup->nReal ){
     struct multiplexReal *p;
-    p = sqlite3_realloc(pGroup->aReal, (iChunk+1)*sizeof(*p));
+    p = sqlite3_realloc64(pGroup->aReal, (iChunk+1)*sizeof(*p));
     if( p==0 ){
       return SQLITE_NOMEM;
     }
@@ -297,11 +267,14 @@ static int multiplexSubFilename(multiplexGroup *pGroup, int iChunk){
   if( pGroup->zName && pGroup->aReal[iChunk].z==0 ){
     char *z;
     int n = pGroup->nName;
-    pGroup->aReal[iChunk].z = z = sqlite3_malloc( n+5 );
+    z = sqlite3_malloc64( n+5 );
     if( z==0 ){
       return SQLITE_NOMEM;
     }
     multiplexFilename(pGroup->zName, pGroup->nName, pGroup->flags, iChunk, z);
+    pGroup->aReal[iChunk].z = (char*)sqlite3_create_filename(z,"","",0,0);
+    sqlite3_free(z);
+    if( pGroup->aReal[iChunk].z==0 ) return SQLITE_NOMEM;
   }
   return SQLITE_OK;
 }
@@ -324,8 +297,8 @@ static sqlite3_file *multiplexSubOpen(
   sqlite3_vfs *pOrigVfs = gMultiplex.pOrigVfs;        /* Real VFS */
 
 #ifdef SQLITE_ENABLE_8_3_NAMES
-  /* If JOURNAL_8_3_OFFSET is set to (say) 400, then any overflow files are 
-  ** part of a database journal are named db.401, db.402, and so on. A 
+  /* If JOURNAL_8_3_OFFSET is set to (say) 400, then any overflow files are
+  ** part of a database journal are named db.401, db.402, and so on. A
   ** database may therefore not grow to larger than 400 chunks. Attempting
   ** to open chunk 401 indicates the database is full. */
   if( iChunk>=SQLITE_MULTIPLEX_JOURNAL_8_3_OFFSET ){
@@ -357,7 +330,7 @@ static sqlite3_file *multiplexSubOpen(
       }
       flags &= ~SQLITE_OPEN_CREATE;
     }
-    pSubOpen = sqlite3_malloc( pOrigVfs->szOsFile );
+    pSubOpen = sqlite3_malloc64( pOrigVfs->szOsFile );
     if( pSubOpen==0 ){
       *rc = SQLITE_IOERR_NOMEM;
       return 0;
@@ -378,8 +351,8 @@ static sqlite3_file *multiplexSubOpen(
 
 /*
 ** Return the size, in bytes, of chunk number iChunk.  If that chunk
-** does not exist, then return 0.  This function does not distingish between
-** non-existant files and zero-length files.
+** does not exist, then return 0.  This function does not distinguish between
+** non-existent files and zero-length files.
 */
 static sqlite3_int64 multiplexSubSize(
   multiplexGroup *pGroup,    /* The multiplexor group */
@@ -394,7 +367,7 @@ static sqlite3_int64 multiplexSubSize(
   if( pSub==0 ) return 0;
   *rc = pSub->pMethods->xFileSize(pSub, &sz);
   return sz;
-}    
+}  
 
 /*
 ** This is the implementation of the multiplex_control() SQL function.
@@ -406,25 +379,25 @@ static void multiplexControlFunc(
 ){
   int rc = SQLITE_OK;
   sqlite3 *db = sqlite3_context_db_handle(context);
-  int op;
+  int op = 0;
   int iVal;
 
-  if( !db || argc!=2 ){ 
-    rc = SQLITE_ERROR; 
+  if( !db || argc!=2 ){
+    rc = SQLITE_ERROR;
   }else{
     /* extract params */
     op = sqlite3_value_int(argv[0]);
     iVal = sqlite3_value_int(argv[1]);
     /* map function op to file_control op */
     switch( op ){
-      case 1: 
-        op = MULTIPLEX_CTRL_ENABLE; 
+      case 1:
+        op = MULTIPLEX_CTRL_ENABLE;
         break;
-      case 2: 
-        op = MULTIPLEX_CTRL_SET_CHUNK_SIZE; 
+      case 2:
+        op = MULTIPLEX_CTRL_SET_CHUNK_SIZE;
         break;
-      case 3: 
-        op = MULTIPLEX_CTRL_SET_MAX_CHUNKS; 
+      case 3:
+        op = MULTIPLEX_CTRL_SET_MAX_CHUNKS;
         break;
       default:
         rc = SQLITE_NOTFOUND;
@@ -438,16 +411,16 @@ static void multiplexControlFunc(
 }
 
 /*
-** This is the entry point to register the auto-extension for the 
+** This is the entry point to register the auto-extension for the
 ** multiplex_control() function.
 */
 static int multiplexFuncInit(
-  sqlite3 *db, 
-  char **pzErrMsg, 
+  sqlite3 *db,
+  char **pzErrMsg,
   const sqlite3_api_routines *pApi
 ){
   int rc;
-  rc = sqlite3_create_function(db, "multiplex_control", 2, SQLITE_ANY, 
+  rc = sqlite3_create_function(db, "multiplex_control", 2, SQLITE_ANY,
       0, multiplexControlFunc, 0, 0);
   return rc;
 }
@@ -468,7 +441,7 @@ static void multiplexSubClose(
     }
     sqlite3_free(pGroup->aReal[iChunk].p);
   }
-  sqlite3_free(pGroup->aReal[iChunk].z);
+  sqlite3_free_filename(pGroup->aReal[iChunk].z);
   memset(&pGroup->aReal[iChunk], 0, sizeof(pGroup->aReal[iChunk]));
 }
 
@@ -516,7 +489,6 @@ static int multiplexOpen(
   /* We need to create a group structure and manage
   ** access to this group of files.
   */
-  multiplexEnter();
   pMultiplexOpen = (multiplexConn*)pConn;
 
   if( rc==SQLITE_OK ){
@@ -524,7 +496,7 @@ static int multiplexOpen(
     nName = zName ? multiplexStrlen30(zName) : 0;
     sz = sizeof(multiplexGroup)                             /* multiplexGroup */
        + nName + 1;                                         /* zName */
-    pGroup = sqlite3_malloc( sz );
+    pGroup = sqlite3_malloc64( sz );
     if( pGroup==0 ){
       rc = SQLITE_NOMEM;
     }
@@ -535,8 +507,8 @@ static int multiplexOpen(
     /* assign pointers to extra space allocated */
     memset(pGroup, 0, sz);
     pMultiplexOpen->pGroup = pGroup;
-    pGroup->bEnabled = -1;
-    pGroup->bTruncate = sqlite3_uri_boolean(zUri, "truncate", 
+    pGroup->bEnabled = (unsigned char)-1;
+    pGroup->bTruncate = (unsigned char)sqlite3_uri_boolean(zUri, "truncate",
                                    (flags & SQLITE_OPEN_MAIN_DB)==0);
     pGroup->szChunk = (int)sqlite3_uri_int64(zUri, "chunksize",
                                         SQLITE_MULTIPLEX_CHUNK_SIZE);
@@ -561,26 +533,29 @@ static int multiplexOpen(
         pGroup->szChunk += 65536;
       }
     }
-    pGroup->flags = flags;
+    pGroup->flags = (flags & ~SQLITE_OPEN_URI);
     rc = multiplexSubFilename(pGroup, 1);
     if( rc==SQLITE_OK ){
       pSubOpen = multiplexSubOpen(pGroup, 0, &rc, pOutFlags, 0);
       if( pSubOpen==0 && rc==SQLITE_OK ) rc = SQLITE_CANTOPEN;
     }
     if( rc==SQLITE_OK ){
-      sqlite3_int64 sz;
+      sqlite3_int64 sz64;
 
-      rc = pSubOpen->pMethods->xFileSize(pSubOpen, &sz);
+      rc = pSubOpen->pMethods->xFileSize(pSubOpen, &sz64);
       if( rc==SQLITE_OK && zName ){
         int bExists;
-        if( sz==0 ){
+        if( flags & SQLITE_OPEN_SUPER_JOURNAL ){
+          pGroup->bEnabled = 0;
+        }else
+        if( sz64==0 ){
           if( flags & SQLITE_OPEN_MAIN_JOURNAL ){
             /* If opening a main journal file and the first chunk is zero
-            ** bytes in size, delete any subsequent chunks from the 
+            ** bytes in size, delete any subsequent chunks from the
             ** file-system. */
             int iChunk = 1;
             do {
-              rc = pOrigVfs->xAccess(pOrigVfs, 
+              rc = pOrigVfs->xAccess(pOrigVfs,
                   pGroup->aReal[iChunk].z, SQLITE_ACCESS_EXISTS, &bExists
               );
               if( rc==SQLITE_OK && bExists ){
@@ -598,16 +573,16 @@ static int multiplexOpen(
           **
           ** Or, if the first overflow file does not exist and the main file is
           ** larger than the chunk size, that means the chunk size is too small.
-          ** But we have no way of determining the intended chunk size, so 
-          ** just disable the multiplexor all togethre.
+          ** But we have no way of determining the intended chunk size, so
+          ** just disable the multiplexor all together.
           */
           rc = pOrigVfs->xAccess(pOrigVfs, pGroup->aReal[1].z,
               SQLITE_ACCESS_EXISTS, &bExists);
           bExists = multiplexSubSize(pGroup, 1, &rc)>0;
-          if( rc==SQLITE_OK && bExists  && sz==(sz&0xffff0000) && sz>0
-              && sz!=pGroup->szChunk ){
-            pGroup->szChunk = (int)sz;
-          }else if( rc==SQLITE_OK && !bExists && sz>pGroup->szChunk ){
+          if( rc==SQLITE_OK && bExists && sz64==(sz64&0xffff0000) && sz64>0
+              && sz64!=pGroup->szChunk ){
+            pGroup->szChunk = (int)sz64;
+          }else if( rc==SQLITE_OK && !bExists && sz64>pGroup->szChunk ){
             pGroup->bEnabled = 0;
           }
         }
@@ -616,20 +591,15 @@ static int multiplexOpen(
 
     if( rc==SQLITE_OK ){
       if( pSubOpen->pMethods->iVersion==1 ){
-        pMultiplexOpen->base.pMethods = &gMultiplex.sIoMethodsV1;
+        pConn->pMethods = &gMultiplex.sIoMethodsV1;
       }else{
-        pMultiplexOpen->base.pMethods = &gMultiplex.sIoMethodsV2;
+        pConn->pMethods = &gMultiplex.sIoMethodsV2;
       }
-      /* place this group at the head of our list */
-      pGroup->pNext = gMultiplex.pGroups;
-      if( gMultiplex.pGroups ) gMultiplex.pGroups->pPrev = pGroup;
-      gMultiplex.pGroups = pGroup;
     }else{
       multiplexFreeComponents(pGroup);
       sqlite3_free(pGroup);
     }
   }
-  multiplexLeave();
   sqlite3_free(zToFree);
   return rc;
 }
@@ -648,11 +618,11 @@ static int multiplexDelete(
   rc = pOrigVfs->xDelete(pOrigVfs, zName, syncDir);
   if( rc==SQLITE_OK ){
     /* If the main chunk was deleted successfully, also delete any subsequent
-    ** chunks - starting with the last (highest numbered). 
+    ** chunks - starting with the last (highest numbered).
     */
     int nName = (int)strlen(zName);
     char *z;
-    z = sqlite3_malloc(nName + 5);
+    z = sqlite3_malloc64(nName + 5);
     if( z==0 ){
       rc = SQLITE_IOERR_NOMEM;
     }else{
@@ -711,7 +681,11 @@ static int multiplexCurrentTime(sqlite3_vfs *a, double *b){
   return gMultiplex.pOrigVfs->xCurrentTime(gMultiplex.pOrigVfs, b);
 }
 static int multiplexGetLastError(sqlite3_vfs *a, int b, char *c){
-  return gMultiplex.pOrigVfs->xGetLastError(gMultiplex.pOrigVfs, b, c);
+  if( gMultiplex.pOrigVfs->xGetLastError ){
+    return gMultiplex.pOrigVfs->xGetLastError(gMultiplex.pOrigVfs, b, c);
+  }else{
+    return 0;
+  }
 }
 static int multiplexCurrentTimeInt64(sqlite3_vfs *a, sqlite3_int64 *b){
   return gMultiplex.pOrigVfs->xCurrentTimeInt64(gMultiplex.pOrigVfs, b);
@@ -721,24 +695,15 @@ static int multiplexCurrentTimeInt64(sqlite3_vfs *a, sqlite3_int64 *b){
 
 /* xClose requests get passed through to the original VFS.
 ** We loop over all open chunk handles and close them.
-** The group structure for this file is unlinked from 
+** The group structure for this file is unlinked from
 ** our list of groups and freed.
 */
 static int multiplexClose(sqlite3_file *pConn){
   multiplexConn *p = (multiplexConn*)pConn;
   multiplexGroup *pGroup = p->pGroup;
   int rc = SQLITE_OK;
-  multiplexEnter();
   multiplexFreeComponents(pGroup);
-  /* remove from linked list */
-  if( pGroup->pNext ) pGroup->pNext->pPrev = pGroup->pPrev;
-  if( pGroup->pPrev ){
-    pGroup->pPrev->pNext = pGroup->pNext;
-  }else{
-    gMultiplex.pGroups = pGroup->pNext;
-  }
   sqlite3_free(pGroup);
-  multiplexLeave();
   return rc;
 }
 
@@ -755,11 +720,8 @@ static int multiplexRead(
   multiplexConn *p = (multiplexConn*)pConn;
   multiplexGroup *pGroup = p->pGroup;
   int rc = SQLITE_OK;
-  int nMutex = 0;
-  multiplexEnter(); nMutex++;
   if( !pGroup->bEnabled ){
     sqlite3_file *pSubOpen = multiplexSubOpen(pGroup, 0, &rc, NULL, 0);
-    multiplexLeave(); nMutex--;
     if( pSubOpen==0 ){
       rc = SQLITE_IOERR_READ;
     }else{
@@ -769,9 +731,7 @@ static int multiplexRead(
     while( iAmt > 0 ){
       int i = (int)(iOfst / pGroup->szChunk);
       sqlite3_file *pSubOpen;
-      if( nMutex==0 ){ multiplexEnter(); nMutex++; }
       pSubOpen = multiplexSubOpen(pGroup, i, &rc, NULL, 1);
-      multiplexLeave(); nMutex--;
       if( pSubOpen ){
         int extra = ((int)(iOfst % pGroup->szChunk) + iAmt) - pGroup->szChunk;
         if( extra<0 ) extra = 0;
@@ -788,8 +748,7 @@ static int multiplexRead(
       }
     }
   }
-  assert( nMutex==0 || nMutex==1 );
-  if( nMutex ) multiplexLeave();
+
   return rc;
 }
 
@@ -806,7 +765,6 @@ static int multiplexWrite(
   multiplexConn *p = (multiplexConn*)pConn;
   multiplexGroup *pGroup = p->pGroup;
   int rc = SQLITE_OK;
-  multiplexEnter();
   if( !pGroup->bEnabled ){
     sqlite3_file *pSubOpen = multiplexSubOpen(pGroup, 0, &rc, NULL, 0);
     if( pSubOpen==0 ){
@@ -831,7 +789,6 @@ static int multiplexWrite(
       }
     }
   }
-  multiplexLeave();
   return rc;
 }
 
@@ -843,7 +800,6 @@ static int multiplexTruncate(sqlite3_file *pConn, sqlite3_int64 size){
   multiplexConn *p = (multiplexConn*)pConn;
   multiplexGroup *pGroup = p->pGroup;
   int rc = SQLITE_OK;
-  multiplexEnter();
   if( !pGroup->bEnabled ){
     sqlite3_file *pSubOpen = multiplexSubOpen(pGroup, 0, &rc, NULL, 0);
     if( pSubOpen==0 ){
@@ -875,7 +831,6 @@ static int multiplexTruncate(sqlite3_file *pConn, sqlite3_int64 size){
     }
     if( rc ) rc = SQLITE_IOERR_TRUNCATE;
   }
-  multiplexLeave();
   return rc;
 }
 
@@ -886,7 +841,6 @@ static int multiplexSync(sqlite3_file *pConn, int flags){
   multiplexGroup *pGroup = p->pGroup;
   int rc = SQLITE_OK;
   int i;
-  multiplexEnter();
   for(i=0; i<pGroup->nReal; i++){
     sqlite3_file *pSubOpen = pGroup->aReal[i].p;
     if( pSubOpen ){
@@ -894,7 +848,6 @@ static int multiplexSync(sqlite3_file *pConn, int flags){
       if( rc2!=SQLITE_OK ) rc = rc2;
     }
   }
-  multiplexLeave();
   return rc;
 }
 
@@ -906,7 +859,6 @@ static int multiplexFileSize(sqlite3_file *pConn, sqlite3_int64 *pSize){
   multiplexGroup *pGroup = p->pGroup;
   int rc = SQLITE_OK;
   int i;
-  multiplexEnter();
   if( !pGroup->bEnabled ){
     sqlite3_file *pSubOpen = multiplexSubOpen(pGroup, 0, &rc, NULL, 0);
     if( pSubOpen==0 ){
@@ -922,7 +874,6 @@ static int multiplexFileSize(sqlite3_file *pConn, sqlite3_int64 *pSize){
       *pSize = i*(sqlite3_int64)pGroup->szChunk + sz;
     }
   }
-  multiplexLeave();
   return rc;
 }
 
@@ -976,7 +927,7 @@ static int multiplexFileControl(sqlite3_file *pConn, int op, void *pArg){
     case MULTIPLEX_CTRL_ENABLE:
       if( pArg ) {
         int bEnabled = *(int *)pArg;
-        pGroup->bEnabled = bEnabled;
+        pGroup->bEnabled = (unsigned char)bEnabled;
         rc = SQLITE_OK;
       }
       break;
@@ -1002,6 +953,92 @@ static int multiplexFileControl(sqlite3_file *pConn, int op, void *pArg){
       /* no-op these */
       rc = SQLITE_OK;
       break;
+    case SQLITE_FCNTL_PRAGMA: {
+      char **aFcntl = (char**)pArg;
+      /*
+      ** EVIDENCE-OF: R-29875-31678 The argument to the SQLITE_FCNTL_PRAGMA
+      ** file control is an array of pointers to strings (char**) in which the
+      ** second element of the array is the name of the pragma and the third
+      ** element is the argument to the pragma or NULL if the pragma has no
+      ** argument.
+      */
+      if( aFcntl[1] && sqlite3_strnicmp(aFcntl[1],"multiplex_",10)==0 ){
+        sqlite3_int64 sz = 0;
+        (void)multiplexFileSize(pConn, &sz);
+        /*
+        ** PRAGMA multiplex_truncate=BOOLEAN;
+        ** PRAGMA multiplex_truncate;
+        **
+        ** Turn the multiplexor truncate feature on or off.  Return either
+        ** "on" or "off" to indicate the new setting.  If the BOOLEAN argument
+        ** is omitted, just return the current value for the truncate setting.
+        */
+        if( sqlite3_stricmp(aFcntl[1],"multiplex_truncate")==0 ){
+          if( aFcntl[2] && aFcntl[2][0] ){
+            if( sqlite3_stricmp(aFcntl[2], "on")==0
+             || sqlite3_stricmp(aFcntl[2], "1")==0 ){
+              pGroup->bTruncate = 1;
+            }else
+            if( sqlite3_stricmp(aFcntl[2], "off")==0
+             || sqlite3_stricmp(aFcntl[2], "0")==0 ){
+              pGroup->bTruncate = 0;
+            }
+          }
+          /* EVIDENCE-OF: R-27806-26076 The handler for an SQLITE_FCNTL_PRAGMA
+          ** file control can optionally make the first element of the char**
+          ** argument point to a string obtained from sqlite3_mprintf() or the
+          ** equivalent and that string will become the result of the pragma
+          ** or the error message if the pragma fails.
+          */
+          aFcntl[0] = sqlite3_mprintf(pGroup->bTruncate ? "on" : "off");
+          rc = SQLITE_OK;
+          break;
+        }
+        /*
+        ** PRAGMA multiplex_enabled;
+        **
+        ** Return 0 or 1 depending on whether the multiplexor is enabled or
+        ** disabled, respectively.
+        */
+        if( sqlite3_stricmp(aFcntl[1],"multiplex_enabled")==0 ){
+          aFcntl[0] = sqlite3_mprintf("%d", pGroup->bEnabled!=0);
+          rc = SQLITE_OK;
+          break;
+        }
+        /*
+        ** PRAGMA multiplex_chunksize;
+        **
+        ** Return the chunksize for the multiplexor, or no-op if the
+        ** multiplexor is not active.
+        */
+        if( sqlite3_stricmp(aFcntl[1],"multiplex_chunksize")==0
+         && pGroup->bEnabled
+        ){
+          aFcntl[0] = sqlite3_mprintf("%u", pGroup->szChunk);
+          rc = SQLITE_OK;
+          break;
+        }
+        /*
+        ** PRAGMA multiplex_filecount;
+        **
+        ** Return the number of disk files currently in use by the
+        ** multiplexor.  This should be the total database size size
+        ** divided by the chunksize and rounded up.
+        */
+        if( sqlite3_stricmp(aFcntl[1],"multiplex_filecount")==0 ){
+          int n = 0;
+          int ii;
+          for(ii=0; ii<pGroup->nReal; ii++){
+            if( pGroup->aReal[ii].p!=0 ) n++;
+          }
+          aFcntl[0] = sqlite3_mprintf("%d", n);
+          rc = SQLITE_OK;
+          break;
+        }
+      }
+      /* If the multiplexor does not handle the pragma, pass it through
+      ** into the default case. */
+    }
     default:
       pSubOpen = multiplexSubOpen(pGroup, 0, &rc, NULL, 0);
       if( pSubOpen ){
@@ -1101,8 +1138,8 @@ static int multiplexShmUnmap(sqlite3_file *pConn, int deleteFlag){
 /*
 ** CAPI: Initialize the multiplex VFS shim - sqlite3_multiplex_initialize()
 **
-** Use the VFS named zOrigVfsName as the VFS that does the actual work.  
-** Use the default if zOrigVfsName==NULL.  
+** Use the VFS named zOrigVfsName as the VFS that does the actual work.
+** Use the default if zOrigVfsName==NULL.
 **
 ** The multiplex VFS shim is named "multiplex".  It will become the default
 ** VFS if makeDefault is non-zero.
@@ -1116,11 +1153,6 @@ int sqlite3_multiplex_initialize(const char *zOrigVfsName, int makeDefault){
   pOrigVfs = sqlite3_vfs_find(zOrigVfsName);
   if( pOrigVfs==0 ) return SQLITE_ERROR;
   assert( pOrigVfs!=&gMultiplex.sThisVfs );
-  gMultiplex.pMutex = sqlite3_mutex_alloc(SQLITE_MUTEX_FAST);
-  if( !gMultiplex.pMutex ){
-    return SQLITE_NOMEM;
-  }
-  gMultiplex.pGroups = NULL;
   gMultiplex.isInitialized = 1;
   gMultiplex.pOrigVfs = pOrigVfs;
   gMultiplex.sThisVfs = *pOrigVfs;
@@ -1162,7 +1194,7 @@ int sqlite3_multiplex_initialize(const char *zOrigVfsName, int makeDefault){
   gMultiplex.sIoMethodsV2.xShmUnmap = multiplexShmUnmap;
   sqlite3_vfs_register(&gMultiplex.sThisVfs, makeDefault);
 
-  sqlite3_auto_extension((void*)multiplexFuncInit);
+  sqlite3_auto_extension((void(*)(void))multiplexFuncInit);
 
   return SQLITE_OK;
 }
@@ -1179,14 +1211,7 @@ int sqlite3_multiplex_initialize(const char *zOrigVfsName, int makeDefault){
 int sqlite3_multiplex_shutdown(int eForce){
   int rc = SQLITE_OK;
   if( gMultiplex.isInitialized==0 ) return SQLITE_MISUSE;
-  if( gMultiplex.pGroups ){
-    sqlite3_log(SQLITE_MISUSE, "sqlite3_multiplex_shutdown() called "
-                "while database connections are still open");
-    if( !eForce ) return SQLITE_MISUSE;
-    rc = SQLITE_MISUSE;
-  }
   gMultiplex.isInitialized = 0;
-  sqlite3_mutex_free(gMultiplex.pMutex);
   sqlite3_vfs_unregister(&gMultiplex.sThisVfs);
   memset(&gMultiplex, 0, sizeof(gMultiplex));
   return rc;
@@ -1194,14 +1219,14 @@ int sqlite3_multiplex_shutdown(int eForce){
 
 /***************************** Test Code ***********************************/
 #ifdef SQLITE_TEST
-#include <tcl.h>
+#include "tclsqlite.h"
 extern const char *sqlite3ErrName(int);
 
 
 /*
 ** tclcmd: sqlite3_multiplex_initialize NAME MAKEDEFAULT
 */
-static int test_multiplex_initialize(
+static int SQLITE_TCLAPI test_multiplex_initialize(
   void * clientData,
   Tcl_Interp *interp,
   int objc,
@@ -1232,7 +1257,7 @@ static int test_multiplex_initialize(
 /*
 ** tclcmd: sqlite3_multiplex_shutdown
 */
-static int test_multiplex_shutdown(
+static int SQLITE_TCLAPI test_multiplex_shutdown(
   void * clientData,
   Tcl_Interp *interp,
   int objc,
@@ -1257,65 +1282,10 @@ static int test_multiplex_shutdown(
   return TCL_OK;
 }
 
-/*
-** tclcmd:  sqlite3_multiplex_dump
-*/
-static int test_multiplex_dump(
-  void * clientData,
-  Tcl_Interp *interp,
-  int objc,
-  Tcl_Obj *CONST objv[]
-){
-  Tcl_Obj *pResult;
-  Tcl_Obj *pGroupTerm;
-  multiplexGroup *pGroup;
-  int i;
-  int nChunks = 0;
-
-  UNUSED_PARAMETER(clientData);
-  UNUSED_PARAMETER(objc);
-  UNUSED_PARAMETER(objv);
-
-  pResult = Tcl_NewObj();
-  multiplexEnter();
-  for(pGroup=gMultiplex.pGroups; pGroup; pGroup=pGroup->pNext){
-    pGroupTerm = Tcl_NewObj();
-
-    if( pGroup->zName ){
-      pGroup->zName[pGroup->nName] = '\0';
-      Tcl_ListObjAppendElement(interp, pGroupTerm,
-          Tcl_NewStringObj(pGroup->zName, -1));
-    }else{
-      Tcl_ListObjAppendElement(interp, pGroupTerm, Tcl_NewObj());
-    }
-    Tcl_ListObjAppendElement(interp, pGroupTerm,
-          Tcl_NewIntObj(pGroup->nName));
-    Tcl_ListObjAppendElement(interp, pGroupTerm,
-          Tcl_NewIntObj(pGroup->flags));
-
-    /* count number of chunks with open handles */
-    for(i=0; i<pGroup->nReal; i++){
-      if( pGroup->aReal[i].p!=0 ) nChunks++;
-    }
-    Tcl_ListObjAppendElement(interp, pGroupTerm,
-          Tcl_NewIntObj(nChunks));
-
-    Tcl_ListObjAppendElement(interp, pGroupTerm,
-          Tcl_NewIntObj(pGroup->szChunk));
-    Tcl_ListObjAppendElement(interp, pGroupTerm,
-          Tcl_NewIntObj(pGroup->nReal));
-
-    Tcl_ListObjAppendElement(interp, pResult, pGroupTerm);
-  }
-  multiplexLeave();
-  Tcl_SetObjResult(interp, pResult);
-  return TCL_OK;
-}
-
 /*
 ** Tclcmd: test_multiplex_control HANDLE DBNAME SUB-COMMAND ?INT-VALUE?
 */
-static int test_multiplex_control(
+static int SQLITE_TCLAPI test_multiplex_control(
   ClientData cd,
   Tcl_Interp *interp,
   int objc,
@@ -1345,8 +1315,8 @@ static int test_multiplex_control(
   }
 
   if( 0==Tcl_GetCommandInfo(interp, Tcl_GetString(objv[1]), &cmdInfo) ){
-    Tcl_AppendResult(interp, "expected database handle, got \"", 0);
-    Tcl_AppendResult(interp, Tcl_GetString(objv[1]), "\"", 0);
+    Tcl_AppendResult(interp, "expected database handle, got \"", NULL);
+    Tcl_AppendResult(interp, Tcl_GetString(objv[1]), "\"", NULL);
     return TCL_ERROR;
   }else{
     db = *(sqlite3 **)cmdInfo.objClientData;
@@ -1386,7 +1356,6 @@ int Sqlitemultiplex_Init(Tcl_Interp *interp){
   } aCmd[] = {
     { "sqlite3_multiplex_initialize", test_multiplex_initialize },
     { "sqlite3_multiplex_shutdown", test_multiplex_shutdown },
-    { "sqlite3_multiplex_dump", test_multiplex_dump },
     { "sqlite3_multiplex_control", test_multiplex_control },
   };
   int i;
