@@ -3,6 +3,8 @@
 */
 #include <stdio.h>
 #include <ctype.h>
+#define ISDIGIT(X) isdigit((unsigned char)(X))
+#define ISPRINT(X) isprint((unsigned char)(X))
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -15,16 +17,24 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <assert.h>
 #include "sqlite3.h"
 
+typedef unsigned char u8;         /* unsigned 8-bit */
+typedef unsigned int u32;         /* unsigned 32-bit */
+typedef sqlite3_int64 i64;        /* signed 64-bit */
+typedef sqlite3_uint64 u64;       /* unsigned 64-bit */
 
-static int pagesize = 1024;     /* Size of a database page */
-static int db = -1;             /* File descriptor for reading the DB */
-static int mxPage = 0;          /* Last page number */
-static int perLine = 16;        /* HEX elements to print per line */
-
-typedef long long int i64;      /* Datatype for 64-bit integers */
 
+static struct GlobalData {
+  i64 pagesize;                   /* Size of a database page */
+  int dbfd;                       /* File descriptor for reading the DB */
+  u32 mxPage;                     /* Last page number */
+  int perLine;                    /* HEX elements to print per line */
+  int bRaw;                       /* True to access db file via OS APIs */
+  sqlite3_file *pFd;              /* File descriptor for non-raw mode */
+  sqlite3 *pDb;                   /* Database handle that owns pFd */
+} g = {1024, -1, 0, 16,   0, 0, 0};
 
 /*
 ** Convert the var-int format into i64.  Return the number of bytes
@@ -45,7 +55,7 @@ static int decodeVarint(const unsigned char *z, i64 *pVal){
 /*
 ** Extract a big-endian 32-bit integer
 */
-static unsigned int decodeInt32(const unsigned char *z){
+static u32 decodeInt32(const u8 *z){
   return (z[0]<<24) + (z[1]<<16) + (z[2]<<8) + z[3];
 }
 
@@ -56,29 +66,129 @@ static void out_of_memory(void){
   exit(1);
 }
 
+/*
+** Open a database connection.
+*/
+static sqlite3 *openDatabase(const char *zPrg, const char *zName){
+  sqlite3 *db = 0;
+  int flags = SQLITE_OPEN_READWRITE | SQLITE_OPEN_URI;
+  int rc = sqlite3_open_v2(zName, &db, flags, 0);
+  if( rc!=SQLITE_OK ){
+    const char *zErr = sqlite3_errmsg(db);
+    fprintf(stderr, "%s: can't open %s (%s)\n", zPrg, zName, zErr);
+    sqlite3_close(db);
+    exit(1);
+  }
+  return db;
+}
+
+/**************************************************************************
+** Beginning of low-level file access functions.
+**
+** All low-level access to the database file read by this program is
+** performed using the following four functions:
+**
+**   fileOpen()     - open the db file
+**   fileClose()    - close the db file
+**   fileRead()     - read raw data from the db file
+**   fileGetsize()  - return the size of the db file in bytes
+*/
+
+/*
+** Open the database file.
+*/
+static void fileOpen(const char *zPrg, const char *zName){
+  assert( g.dbfd<0 );
+  if( g.bRaw==0 ){
+    int rc;
+    void *pArg = (void *)(&g.pFd);
+    g.pDb = openDatabase(zPrg, zName);
+    rc = sqlite3_file_control(g.pDb, "main", SQLITE_FCNTL_FILE_POINTER, pArg);
+    if( rc!=SQLITE_OK ){
+      fprintf(stderr, 
+          "%s: failed to obtain fd for %s (SQLite too old?)\n", zPrg, zName
+      );
+      exit(1);
+    }
+  }else{
+    g.dbfd = open(zName, O_RDONLY);
+    if( g.dbfd<0 ){
+      fprintf(stderr,"%s: can't open %s\n", zPrg, zName);
+      exit(1);
+    }
+  }
+}
+
+/*
+** Close the database file opened by fileOpen()
+*/
+static void fileClose(){
+  if( g.bRaw==0 ){
+    sqlite3_close(g.pDb);
+    g.pDb = 0;
+    g.pFd = 0;
+  }else{
+    close(g.dbfd);
+    g.dbfd = -1;
+  }
+}
+
 /*
 ** Read content from the file.
 **
-** Space to hold the content is obtained from malloc() and needs to be
-** freed by the caller.
+** Space to hold the content is obtained from sqlite3_malloc() and needs 
+** to be freed by the caller.
 */
-static unsigned char *getContent(int ofst, int nByte){
+static unsigned char *fileRead(sqlite3_int64 ofst, int nByte){
   unsigned char *aData;
-  aData = malloc(nByte+32);
+  int got;
+  aData = sqlite3_malloc64(32+(i64)nByte);
   if( aData==0 ) out_of_memory();
   memset(aData, 0, nByte+32);
-  lseek(db, ofst, SEEK_SET);
-  if( read(db, aData, nByte)<nByte ) memset(aData, 0, nByte);
+  if( g.bRaw==0 ){
+    int rc = g.pFd->pMethods->xRead(g.pFd, (void*)aData, nByte, ofst);
+    if( rc!=SQLITE_OK && rc!=SQLITE_IOERR_SHORT_READ ){
+      fprintf(stderr, "error in xRead() - %d\n", rc);
+      exit(1);
+    }
+  }else{
+    lseek(g.dbfd, (long)ofst, SEEK_SET);
+    got = read(g.dbfd, aData, nByte);
+    if( got>0 && got<nByte ) memset(aData+got, 0, nByte-got);
+  }
   return aData;
 }
 
+/*
+** Return the size of the file in byte.
+*/
+static i64 fileGetsize(void){
+  i64 res = 0;
+  if( g.bRaw==0 ){
+    int rc = g.pFd->pMethods->xFileSize(g.pFd, &res);
+    if( rc!=SQLITE_OK ){
+      fprintf(stderr, "error in xFileSize() - %d\n", rc);
+      exit(1);
+    }
+  }else{
+    struct stat sbuf;
+    fstat(g.dbfd, &sbuf);
+    res = (sqlite3_int64)(sbuf.st_size);
+  }
+  return res;
+}
+
+/*
+** End of low-level file access functions.
+**************************************************************************/
+
 /*
 ** Print a range of bytes as hex and as ascii.
 */
 static unsigned char *print_byte_range(
-  int ofst,          /* First byte in the range of bytes to print */
-  int nByte,         /* Number of bytes to print */
-  int printOfst      /* Add this amount to the index on the left column */
+  sqlite3_int64 ofst,  /* First byte in the range of bytes to print */
+  int nByte,           /* Number of bytes to print */
+  int printOfst        /* Add this amount to the index on the left column */
 ){
   unsigned char *aData;
   int i, j;
@@ -96,21 +206,27 @@ static unsigned char *print_byte_range(
     zOfstFmt = " %08x: ";
   }
 
-  aData = getContent(ofst, nByte);
-  for(i=0; i<nByte; i += perLine){
+  aData = fileRead(ofst, nByte);
+  for(i=0; i<nByte; i += g.perLine){
+    int go = 0;
+    for(j=0; j<g.perLine; j++){
+      if( i+j>nByte ){ break; }
+      if( aData[i+j] ){ go = 1; break; }
+    }
+    if( !go && i>0 && i+g.perLine<nByte ) continue;
     fprintf(stdout, zOfstFmt, i+printOfst);
-    for(j=0; j<perLine; j++){
+    for(j=0; j<g.perLine; j++){
       if( i+j>nByte ){
         fprintf(stdout, "   ");
       }else{
         fprintf(stdout,"%02x ", aData[i+j]);
       }
     }
-    for(j=0; j<perLine; j++){
+    for(j=0; j<g.perLine; j++){
       if( i+j>nByte ){
         fprintf(stdout, " ");
       }else{
-        fprintf(stdout,"%c", isprint(aData[i+j]) ? aData[i+j] : '.');
+        fprintf(stdout,"%c", ISPRINT(aData[i+j]) ? aData[i+j] : '.');
       }
     }
     fprintf(stdout,"\n");
@@ -121,18 +237,18 @@ static unsigned char *print_byte_range(
 /*
 ** Print an entire page of content as hex
 */
-static void print_page(int iPg){
-  int iStart;
+static void print_page(u32 iPg){
+  i64 iStart;
   unsigned char *aData;
-  iStart = (iPg-1)*pagesize;
-  fprintf(stdout, "Page %d:   (offsets 0x%x..0x%x)\n",
-          iPg, iStart, iStart+pagesize-1);
-  aData = print_byte_range(iStart, pagesize, 0);
-  free(aData);
+  iStart = ((i64)(iPg-1))*g.pagesize;
+  fprintf(stdout, "Page %u:   (offsets 0x%llx..0x%llx)\n",
+          iPg, iStart, iStart+g.pagesize-1);
+  aData = print_byte_range(iStart, g.pagesize, 0);
+  sqlite3_free(aData);
 }
 
 
-/* Print a line of decode output showing a 4-byte integer.
+/* Print a line of decoded output showing a 4-byte unsigned integer.
 */
 static void print_decode_line(
   unsigned char *aData,      /* Content being decoded */
@@ -140,7 +256,7 @@ static void print_decode_line(
   const char *zMsg           /* Message to append */
 ){
   int i, j;
-  int val = aData[ofst];
+  u32 val = aData[ofst];
   char zBuf[100];
   sprintf(zBuf, " %03x: %02x", ofst, aData[ofst]);
   i = (int)strlen(zBuf);
@@ -153,7 +269,7 @@ static void print_decode_line(
     }
     i += (int)strlen(&zBuf[i]);
   }
-  sprintf(&zBuf[i], "   %9d", val);
+  sprintf(&zBuf[i], "   %10u", val);
   printf("%s  %s\n", zBuf, zMsg);
 }
 
@@ -187,6 +303,7 @@ static void print_db_header(void){
   print_decode_line(aData, 88, 4, "meta[12]");
   print_decode_line(aData, 92, 4, "Change counter for version number");
   print_decode_line(aData, 96, 4, "SQLite version number");
+  sqlite3_free(aData);
 }
 
 /*
@@ -265,14 +382,14 @@ static i64 localPayload(i64 nPayload, char cType){
   i64 nLocal;
   if( cType==13 ){
     /* Table leaf */
-    maxLocal = pagesize-35;
-    minLocal = (pagesize-12)*32/255-23;
+    maxLocal = g.pagesize-35;
+    minLocal = (g.pagesize-12)*32/255-23;
   }else{
-    maxLocal = (pagesize-12)*64/255-23;
-    minLocal = (pagesize-12)*32/255-23;
+    maxLocal = (g.pagesize-12)*64/255-23;
+    minLocal = (g.pagesize-12)*32/255-23;
   }
   if( nPayload>maxLocal ){
-    surplus = minLocal + (nPayload-minLocal)%(pagesize-4);
+    surplus = minLocal + (nPayload-minLocal)%(g.pagesize-4);
     if( surplus<=maxLocal ){
       nLocal = surplus;
     }else{
@@ -299,7 +416,7 @@ static i64 describeCell(
   int i;
   i64 nDesc = 0;
   int n = 0;
-  int leftChild;
+  u32 leftChild;
   i64 nPayload;
   i64 rowid;
   i64 nLocal;
@@ -309,7 +426,7 @@ static i64 describeCell(
     leftChild = ((a[0]*256 + a[1])*256 + a[2])*256 + a[3];
     a += 4;
     n += 4;
-    sprintf(zDesc, "lx: %d ", leftChild);
+    sprintf(zDesc, "lx: %u ", leftChild);
     nDesc = strlen(zDesc);
   }
   if( cType!=5 ){
@@ -330,10 +447,10 @@ static i64 describeCell(
     nDesc += strlen(&zDesc[nDesc]);
   }
   if( nLocal<nPayload ){
-    int ovfl;
+    u32 ovfl;
     unsigned char *b = &a[nLocal];
     ovfl = ((b[0]*256 + b[1])*256 + b[2])*256 + b[3];
-    sprintf(&zDesc[nDesc], "ov: %d ", ovfl);
+    sprintf(&zDesc[nDesc], "ov: %u ", ovfl);
     nDesc += strlen(&zDesc[nDesc]);
     n += 4;
   }
@@ -375,8 +492,8 @@ static void decodeCell(
   int szPgHdr,            /* Size of the page header.  0 or 100 */
   int ofst                /* Cell begins at a[ofst] */
 ){
-  int i, j;
-  int leftChild;
+  int i, j = 0;
+  u32 leftChild;
   i64 k;
   i64 nPayload;
   i64 rowid;
@@ -395,7 +512,7 @@ static void decodeCell(
   if( cType<=5 ){
     leftChild = ((x[0]*256 + x[1])*256 + x[2])*256 + x[3];
     printBytes(a, x, 4);
-    printf("left child page:: %d\n", leftChild);
+    printf("left child page:: %u\n", leftChild);
     x += 4;
   }
   if( cType!=5 ){
@@ -426,7 +543,7 @@ static void decodeCell(
     j = i;
     nCol = 0;
     k = nHdr;
-    while( x+j<end && j<nHdr ){
+    while( x+j<=end && j<nHdr ){
        const char *zTypeName;
        int sz = 0;
        char zNm[30];
@@ -493,7 +610,7 @@ static void decodeCell(
          }else{
            zConst[0] = '\'';
            for(ii=1, jj=0; jj<szCol[i] && ii<24; jj++, ii++){
-             zConst[ii] = isprint(pData[jj]) ? pData[jj] : '.';
+             zConst[ii] = ISPRINT(pData[jj]) ? pData[jj] : '.';
            }
            zConst[ii] = 0;
          }
@@ -513,7 +630,7 @@ static void decodeCell(
   }
   if( nLocal<nPayload ){
     printBytes(a, x+nLocal, 4);
-    printf("overflow-page: %d\n", decodeInt32(x+nLocal));
+    printf("overflow-page: %u\n", decodeInt32(x+nLocal));
   }
 }
 
@@ -546,11 +663,11 @@ static void decode_btree_page(
       case 'c': showCellContent = 1;  break;
       case 'm': showMap = 1;          break;
       case 'd': {
-        if( !isdigit(zArgs[1]) ){
+        if( !ISDIGIT(zArgs[1]) ){
           cellToDecode = -1;
         }else{
           cellToDecode = 0;
-          while( isdigit(zArgs[1]) ){
+          while( ISDIGIT(zArgs[1]) ){
             zArgs++;
             cellToDecode = cellToDecode*10 + zArgs[0] - '0';
           }
@@ -579,8 +696,8 @@ static void decode_btree_page(
     printf(" key: lx=left-child n=payload-size r=rowid\n");
   }
   if( showMap ){
-    zMap = malloc(pagesize);
-    memset(zMap, '.', pagesize);
+    zMap = sqlite3_malloc(g.pagesize);
+    memset(zMap, '.', g.pagesize);
     memset(zMap, '1', hdrSize);
     memset(&zMap[hdrSize], 'H', iCellPtr);
     memset(&zMap[hdrSize+iCellPtr], 'P', 2*nCell);
@@ -609,10 +726,10 @@ static void decode_btree_page(
   }
   if( showMap ){
     printf("Page map:  (H=header P=cell-index 1=page-1-header .=free-space)\n");
-    for(i=0; i<pagesize; i+=64){
+    for(i=0; (u32)i<g.pagesize; i+=64){
       printf(" %03x: %.64s\n", i, &zMap[i]);
     }
-    free(zMap);
+    sqlite3_free(zMap);
   }
 }
 
@@ -620,23 +737,23 @@ static void decode_btree_page(
 ** Decode a freelist trunk page.
 */
 static void decode_trunk_page(
-  int pgno,             /* The page number */
-  int pagesize,         /* Size of each page */
+  u32 pgno,             /* The page number */
   int detail,           /* Show leaf pages if true */
   int recursive         /* Follow the trunk change if true */
 ){
-  int n, i;
+  u32 i;
+  u32 n;
   unsigned char *a;
   while( pgno>0 ){
-    a = getContent((pgno-1)*pagesize, pagesize);
+    a = fileRead((pgno-1)*g.pagesize, g.pagesize);
     printf("Decode of freelist trunk page %d:\n", pgno);
     print_decode_line(a, 0, 4, "Next freelist trunk page");
     print_decode_line(a, 4, 4, "Number of entries on this page");
     if( detail ){
-      n = (int)decodeInt32(&a[4]);
-      for(i=0; i<n; i++){
-        unsigned int x = decodeInt32(&a[8+4*i]);
-        char zIdx[10];
+      n = decodeInt32(&a[4]);
+      for(i=0; i<n && i<g.pagesize/4; i++){
+        u32 x = decodeInt32(&a[8+4*i]);
+        char zIdx[13];
         sprintf(zIdx, "[%d]", i);
         printf("  %5s %7u", zIdx, x);
         if( i%5==4 ) printf("\n");
@@ -646,9 +763,9 @@ static void decode_trunk_page(
     if( !recursive ){
       pgno = 0;
     }else{
-      pgno = (int)decodeInt32(&a[0]);
+      pgno = decodeInt32(&a[0]);
     }
-    free(a);
+    sqlite3_free(a);
   }
 }
 
@@ -660,16 +777,16 @@ static char **zPageUse;
 /*
 ** Add a comment on the use of a page.
 */
-static void page_usage_msg(int pgno, const char *zFormat, ...){
+static void page_usage_msg(u32 pgno, const char *zFormat, ...){
   va_list ap;
   char *zMsg;
 
   va_start(ap, zFormat);
   zMsg = sqlite3_vmprintf(zFormat, ap);
   va_end(ap);
-  if( pgno<=0 || pgno>mxPage ){
-    printf("ERROR: page %d out of range 1..%d: %s\n",
-            pgno, mxPage, zMsg);
+  if( pgno<=0 || pgno>g.mxPage ){
+    printf("ERROR: page %d out of range 1..%u: %s\n",
+            pgno, g.mxPage, zMsg);
     sqlite3_free(zMsg);
     return;
   }
@@ -688,7 +805,7 @@ static void page_usage_msg(int pgno, const char *zFormat, ...){
 static void page_usage_cell(
   unsigned char cType,    /* Page type */
   unsigned char *a,       /* Cell content */
-  int pgno,               /* page containing the cell */
+  u32 pgno,               /* page containing the cell */
   int cellno              /* Index of the cell on the page */
 ){
   int i;
@@ -715,24 +832,35 @@ static void page_usage_cell(
     n += i;
   }
   if( nLocal<nPayload ){
-    int ovfl = decodeInt32(a+nLocal);
-    int cnt = 0;
-    while( ovfl && (cnt++)<mxPage ){
-      page_usage_msg(ovfl, "overflow %d from cell %d of page %d",
+    u32 ovfl = decodeInt32(a+nLocal);
+    u32 cnt = 0;
+    while( ovfl && (cnt++)<g.mxPage ){
+      page_usage_msg(ovfl, "overflow %d from cell %d of page %u",
                      cnt, cellno, pgno);
-      a = getContent((ovfl-1)*pagesize, 4);
+      a = fileRead((ovfl-1)*(sqlite3_int64)g.pagesize, 4);
       ovfl = decodeInt32(a);
-      free(a);
+      sqlite3_free(a);
     }
   }
 }
 
+/*
+** True if the memory is all zeros
+*/
+static int allZero(unsigned char *a, int n){
+  while( n && (a++)[0]==0 ){ n--; }
+  return n==0;
+}
+
 
 /*
-** Describe the usages of a b-tree page
+** Describe the usages of a b-tree page.
+**
+** If parent==0, then this is the root of a btree.  If parent<0 then
+** this is an orphan page.
 */
 static void page_usage_btree(
-  int pgno,             /* Page to describe */
+  u32 pgno,             /* Page to describe */
   int parent,           /* Parent of this page.  0 for root pages */
   int idx,              /* Which child of the parent */
   const char *zName     /* Name of the table */
@@ -742,30 +870,61 @@ static void page_usage_btree(
   int nCell;
   int i;
   int hdr = pgno==1 ? 100 : 0;
+  char zEntry[30];
 
-  if( pgno<=0 || pgno>mxPage ) return;
-  a = getContent((pgno-1)*pagesize, pagesize);
+  if( pgno<=0 || pgno>g.mxPage ) return;
+  a = fileRead((pgno-1)*g.pagesize, g.pagesize);
   switch( a[hdr] ){
+    case 0: {
+      if( allZero(a, g.pagesize) ){
+        zType = "zeroed page";
+      }else if( parent<0 ){
+        return;
+      }else{
+        zType = "corrupt node";
+      }
+      break;
+    }
     case 2:  zType = "interior node of index";  break;
     case 5:  zType = "interior node of table";  break;
     case 10: zType = "leaf of index";           break;
     case 13: zType = "leaf of table";           break;
-  }
-  if( parent ){
-    page_usage_msg(pgno, "%s [%s], child %d of page %d",
-                   zType, zName, idx, parent);
-  }else{
-    page_usage_msg(pgno, "root %s [%s]", zType, zName);
+    default: {
+      if( parent<0 ) return;
+      zType = "corrupt node";
+    }
   }
   nCell = a[hdr+3]*256 + a[hdr+4];
+  if( nCell==1 ){
+    sqlite3_snprintf(sizeof(zEntry),zEntry,"1 row");
+  }else{
+    sqlite3_snprintf(sizeof(zEntry),zEntry,"%d rows", nCell);
+  }
+  if( parent>0 ){
+    page_usage_msg(pgno, "%s [%s], child %d of page %d, %s",
+                   zType, zName, idx, parent, zEntry);
+  }else if( parent==0 ){
+    page_usage_msg(pgno, "root %s [%s], %s", zType, zName, zEntry);
+  }else{
+    page_usage_msg(pgno, "orphaned %s, %s", zType, zEntry);
+  }
   if( a[hdr]==2 || a[hdr]==5 ){
     int cellstart = hdr+12;
-    unsigned int child;
+    u32 child;
     for(i=0; i<nCell; i++){
-      int ofst;
+      u32 cellidx;
+      u32 ofst;
 
-      ofst = cellstart + i*2;
-      ofst = a[ofst]*256 + a[ofst+1];
+      cellidx = cellstart + i*2;
+      if( cellidx+1 >= g.pagesize ){
+        printf("ERROR: page %d too many cells (%d)\n", pgno, nCell);
+        break;
+      }
+      ofst = a[cellidx]*256 + a[cellidx+1];
+      if( ofst<cellidx+2 || ofst+4>=g.pagesize ){
+        printf("ERROR: page %d cell %d out of bounds\n", pgno, i);
+        continue;
+      }
       child = decodeInt32(a+ofst);
       page_usage_btree(child, pgno, i, zName);
     }
@@ -781,13 +940,13 @@ static void page_usage_btree(
       page_usage_cell(a[hdr], a+ofst, pgno, i);
     }
   }
-  free(a);
+  sqlite3_free(a);
 }
 
 /*
 ** Determine page usage by the freelist
 */
-static void page_usage_freelist(int pgno){
+static void page_usage_freelist(u32 pgno){
   unsigned char *a;
   int cnt = 0;
   int i;
@@ -795,17 +954,21 @@ static void page_usage_freelist(int pgno){
   int iNext;
   int parent = 1;
 
-  while( pgno>0 && pgno<=mxPage && (cnt++)<mxPage ){
+  while( pgno>0 && pgno<=g.mxPage && (u32)(cnt++)<g.mxPage ){
     page_usage_msg(pgno, "freelist trunk #%d child of %d", cnt, parent);
-    a = getContent((pgno-1)*pagesize, pagesize);
+    a = fileRead((pgno-1)*g.pagesize, g.pagesize);
     iNext = decodeInt32(a);
     n = decodeInt32(a+4);
+    if( n>(g.pagesize - 8)/4 ){
+      printf("ERROR: page %d too many freelist entries (%d)\n", pgno, n);
+      n = (g.pagesize - 8)/4;
+    }
     for(i=0; i<n; i++){
       int child = decodeInt32(a + (i*4+8));
       page_usage_msg(child, "freelist leaf, child %d of trunk page %d",
                      i, pgno);
     }
-    free(a);
+    sqlite3_free(a);
     parent = pgno;
     pgno = iNext;
   }
@@ -814,13 +977,13 @@ static void page_usage_freelist(int pgno){
 /*
 ** Determine pages used as PTRMAP pages
 */
-static void page_usage_ptrmap(unsigned char *a){
-  if( a[55] ){
-    int usable = pagesize - a[20];
-    int pgno = 2;
+static void page_usage_ptrmap(u8 *a){
+  if( decodeInt32(a+52) ){
+    int usable = g.pagesize - a[20];
+    u64 pgno = 2;
     int perPage = usable/5;
-    while( pgno<=mxPage ){
-      page_usage_msg(pgno, "PTRMAP page covering %d..%d",
+    while( pgno<=g.mxPage ){
+      page_usage_msg((u32)pgno, "PTRMAP page covering %llu..%llu",
                            pgno+1, pgno+perPage);
       pgno += perPage + 1;
     }
@@ -830,8 +993,8 @@ static void page_usage_ptrmap(unsigned char *a){
 /*
 ** Try to figure out how every page in the database file is being used.
 */
-static void page_usage_report(const char *zDbName){
-  int i, j;
+static void page_usage_report(const char *zPrg, const char *zDbName){
+  u32 i, j;
   int rc;
   sqlite3 *db;
   sqlite3_stmt *pStmt;
@@ -839,31 +1002,26 @@ static void page_usage_report(const char *zDbName){
   char zQuery[200];
 
   /* Avoid the pathological case */
-  if( mxPage<1 ){
+  if( g.mxPage<1 ){
     printf("empty database\n");
     return;
   }
 
   /* Open the database file */
-  rc = sqlite3_open(zDbName, &db);
-  if( rc ){
-    printf("cannot open database: %s\n", sqlite3_errmsg(db));
-    sqlite3_close(db);
-    return;
-  }
+  db = openDatabase(zPrg, zDbName);
 
-  /* Set up global variables zPageUse[] and mxPage to record page
+  /* Set up global variables zPageUse[] and g.mxPage to record page
   ** usages */
-  zPageUse = sqlite3_malloc( sizeof(zPageUse[0])*(mxPage+1) );
+  zPageUse = sqlite3_malloc64( sizeof(zPageUse[0])*(g.mxPage+1) );
   if( zPageUse==0 ) out_of_memory();
-  memset(zPageUse, 0, sizeof(zPageUse[0])*(mxPage+1));
+  memset(zPageUse, 0, sizeof(zPageUse[0])*(g.mxPage+1));
 
   /* Discover the usage of each page */
-  a = getContent(0, 100);
+  a = fileRead(0, 100);
   page_usage_freelist(decodeInt32(a+32));
   page_usage_ptrmap(a);
-  free(a);
-  page_usage_btree(1, 0, 0, "sqlite_master");
+  sqlite3_free(a);
+  page_usage_btree(1, 0, 0, "sqlite_schema");
   sqlite3_exec(db, "PRAGMA writable_schema=ON", 0, 0, 0);
   for(j=0; j<2; j++){
     sqlite3_snprintf(sizeof(zQuery), zQuery,
@@ -872,7 +1030,7 @@ static void page_usage_report(const char *zDbName){
     rc = sqlite3_prepare_v2(db, zQuery, -1, &pStmt, 0);
     if( rc==SQLITE_OK ){
       while( sqlite3_step(pStmt)==SQLITE_ROW ){
-        int pgno = sqlite3_column_int(pStmt, 2);
+        u32 pgno = (u32)sqlite3_column_int64(pStmt, 2);
         page_usage_btree(pgno, 0, 0, (const char*)sqlite3_column_text(pStmt,1));
       }
     }else{
@@ -884,8 +1042,11 @@ static void page_usage_report(const char *zDbName){
   sqlite3_close(db);
 
   /* Print the report and free memory used */
-  for(i=1; i<=mxPage; i++){
-    printf("%5d: %s\n", i, zPageUse[i] ? zPageUse[i] : "???");
+  for(i=1; i<=g.mxPage; i++){
+    if( zPageUse[i]==0 ) page_usage_btree(i, -1, 0, 0);
+    printf("%5u: %s\n", i, zPageUse[i] ? zPageUse[i] : "???");
+  }
+  for(i=1; i<=g.mxPage; i++){
     sqlite3_free(zPageUse[i]);
   }
   sqlite3_free(zPageUse);
@@ -896,7 +1057,7 @@ static void page_usage_report(const char *zDbName){
 ** Try to figure out how every page in the database file is being used.
 */
 static void ptrmap_coverage_report(const char *zDbName){
-  int pgno;
+  u64 pgno;
   unsigned char *aHdr;
   unsigned char *a;
   int usable;
@@ -904,38 +1065,61 @@ static void ptrmap_coverage_report(const char *zDbName){
   int i;
 
   /* Avoid the pathological case */
-  if( mxPage<1 ){
+  if( g.mxPage<1 ){
     printf("empty database\n");
     return;
   }
 
   /* Make sure PTRMAPs are used in this database */
-  aHdr = getContent(0, 100);
+  aHdr = fileRead(0, 100);
   if( aHdr[55]==0 ){
     printf("database does not use PTRMAP pages\n");
     return;
   }
-  usable = pagesize - aHdr[20];
+  usable = g.pagesize - aHdr[20];
   perPage = usable/5;
-  free(aHdr);
-  printf("%5d: root of sqlite_master\n", 1);
-  for(pgno=2; pgno<=mxPage; pgno += perPage+1){
-    printf("%5d: PTRMAP page covering %d..%d\n", pgno,
+  sqlite3_free(aHdr);
+  printf("%5d: root of sqlite_schema\n", 1);
+  for(pgno=2; pgno<=g.mxPage; pgno += perPage+1){
+    printf("%5llu: PTRMAP page covering %llu..%llu\n", pgno,
            pgno+1, pgno+perPage);
-    a = getContent((pgno-1)*pagesize, usable);
-    for(i=0; i+5<=usable && pgno+1+i/5<=mxPage; i+=5){
-      const char *zType = "???";
-      unsigned int iFrom = decodeInt32(&a[i+1]);
+    a = fileRead((pgno-1)*g.pagesize, usable);
+    for(i=0; i+5<=usable; i+=5){
+      const char *zType;
+      u32 iFrom = decodeInt32(&a[i+1]);
+      const char *zExtra = pgno+1+i/5>g.mxPage ? " (off end of DB)" : "";
       switch( a[i] ){
         case 1:  zType = "b-tree root page";        break;
         case 2:  zType = "freelist page";           break;
         case 3:  zType = "first page of overflow";  break;
         case 4:  zType = "later page of overflow";  break;
         case 5:  zType = "b-tree non-root page";    break;
+        default: {
+          if( zExtra[0]==0 ){
+            printf("%5llu: invalid (0x%02x), parent=%u\n", 
+                   pgno+1+i/5, a[i], iFrom);
+          }
+          zType = 0;
+          break;
+        }
+      }
+      if( zType ){
+        printf("%5llu: %s, parent=%u%s\n", pgno+1+i/5, zType, iFrom, zExtra);
       }
-      printf("%5d: %s, parent=%u\n", pgno+1+i/5, zType, iFrom);
     }
-    free(a);
+    sqlite3_free(a);
+  }
+}
+
+/*
+** Check the range validity for a page number.  Print an error and
+** exit if the page is out of range.
+*/
+static void checkPageValidity(int iPage){
+  if( iPage<1 || iPage>g.mxPage ){
+    fprintf(stderr, "Invalid page number %d:  valid range is 1..%d\n",
+            iPage, g.mxPage);
+    exit(1);
   }
 }
 
@@ -943,8 +1127,10 @@ static void ptrmap_coverage_report(const char *zDbName){
 ** Print a usage comment
 */
 static void usage(const char *argv0){
-  fprintf(stderr, "Usage %s FILENAME ?args...?\n\n", argv0);
+  fprintf(stderr, "Usage %s ?--uri? FILENAME ?args...?\n\n", argv0);
   fprintf(stderr,
+    "switches:\n"
+    "    --raw           Read db file directly, bypassing SQLite VFS\n"
     "args:\n"
     "    dbheader        Show database header\n"
     "    pgidx           Index of how each page is used\n"
@@ -957,97 +1143,113 @@ static void usage(const char *argv0){
     "    NNNbdCCC        Decode cell CCC on btree page NNN\n"
     "    NNNt            Decode freelist trunk page NNN\n"
     "    NNNtd           Show leaf freelist pages on the decode\n"
-    "    NNNtr           Recurisvely decode freelist starting at NNN\n"
+    "    NNNtr           Recursively decode freelist starting at NNN\n"
   );
 }
 
 int main(int argc, char **argv){
-  struct stat sbuf;
-  unsigned char zPgSz[2];
-  if( argc<2 ){
-    usage(argv[0]);
+  sqlite3_int64 szFile;
+  unsigned char *zPgSz;
+  const char *zPrg = argv[0];     /* Name of this executable */
+  char **azArg = argv;
+  int nArg = argc;
+
+  /* Check for the "--uri" or "-uri" switch. */
+  if( nArg>1 ){
+    if( sqlite3_stricmp("-raw", azArg[1])==0 
+     || sqlite3_stricmp("--raw", azArg[1])==0
+    ){
+      g.bRaw = 1;
+      azArg++;
+      nArg--;
+    }
+  }
+
+  if( nArg<2 ){
+    usage(zPrg);
     exit(1);
   }
-  db = open(argv[1], O_RDONLY);
-  if( db<0 ){
-    fprintf(stderr,"%s: can't open %s\n", argv[0], argv[1]);
-    exit(1);
-  }
-  zPgSz[0] = 0;
-  zPgSz[1] = 0;
-  lseek(db, 16, SEEK_SET);
-  if( read(db, zPgSz, 2)<2 ) memset(zPgSz, 0, 2);
-  pagesize = zPgSz[0]*256 + zPgSz[1]*65536;
-  if( pagesize==0 ) pagesize = 1024;
-  printf("Pagesize: %d\n", pagesize);
-  fstat(db, &sbuf);
-  mxPage = sbuf.st_size/pagesize;
-  printf("Available pages: 1..%d\n", mxPage);
-  if( argc==2 ){
-    int i;
-    for(i=1; i<=mxPage; i++) print_page(i);
+
+  fileOpen(zPrg, azArg[1]);
+  szFile = fileGetsize();
+
+  zPgSz = fileRead(16, 2);
+  g.pagesize = zPgSz[0]*256 + zPgSz[1]*65536;
+  if( g.pagesize==0 ) g.pagesize = 1024;
+  sqlite3_free(zPgSz);
+
+  printf("Pagesize: %d\n", (int)g.pagesize);
+  g.mxPage = (u32)((szFile+g.pagesize-1)/g.pagesize);
+
+  printf("Available pages: 1..%u\n", g.mxPage);
+  if( nArg==2 ){
+    u32 i;
+    for(i=1; i<=g.mxPage; i++) print_page(i);
   }else{
     int i;
-    for(i=2; i<argc; i++){
-      int iStart, iEnd;
+    for(i=2; i<nArg; i++){
+      u32 iStart, iEnd;
       char *zLeft;
-      if( strcmp(argv[i], "dbheader")==0 ){
+      if( strcmp(azArg[i], "dbheader")==0 ){
         print_db_header();
         continue;
       }
-      if( strcmp(argv[i], "pgidx")==0 ){
-        page_usage_report(argv[1]);
+      if( strcmp(azArg[i], "pgidx")==0 ){
+        page_usage_report(zPrg, azArg[1]);
         continue;
       }
-      if( strcmp(argv[i], "ptrmap")==0 ){
-        ptrmap_coverage_report(argv[1]);
+      if( strcmp(azArg[i], "ptrmap")==0 ){
+        ptrmap_coverage_report(azArg[1]);
         continue;
       }
-      if( strcmp(argv[i], "help")==0 ){
-        usage(argv[0]);
+      if( strcmp(azArg[i], "help")==0 ){
+        usage(zPrg);
         continue;
       }
-      if( !isdigit(argv[i][0]) ){
-        fprintf(stderr, "%s: unknown option: [%s]\n", argv[0], argv[i]);
+      if( !ISDIGIT(azArg[i][0]) ){
+        fprintf(stderr, "%s: unknown option: [%s]\n", zPrg, azArg[i]);
         continue;
       }
-      iStart = strtol(argv[i], &zLeft, 0);
+      iStart = strtoul(azArg[i], &zLeft, 0);
+      checkPageValidity(iStart);
       if( zLeft && strcmp(zLeft,"..end")==0 ){
-        iEnd = mxPage;
+        iEnd = g.mxPage;
       }else if( zLeft && zLeft[0]=='.' && zLeft[1]=='.' ){
         iEnd = strtol(&zLeft[2], 0, 0);
+        checkPageValidity(iEnd);
       }else if( zLeft && zLeft[0]=='b' ){
-        int ofst, nByte, hdrSize;
+        i64 ofst;
+        int nByte, hdrSize;
         unsigned char *a;
         if( iStart==1 ){
           ofst = hdrSize = 100;
-          nByte = pagesize-100;
+          nByte = g.pagesize-100;
         }else{
           hdrSize = 0;
-          ofst = (iStart-1)*pagesize;
-          nByte = pagesize;
+          ofst = (iStart-1)*g.pagesize;
+          nByte = g.pagesize;
         }
-        a = getContent(ofst, nByte);
+        a = fileRead(ofst, nByte);
         decode_btree_page(a, iStart, hdrSize, &zLeft[1]);
-        free(a);
+        sqlite3_free(a);
         continue;
       }else if( zLeft && zLeft[0]=='t' ){
         int detail = 0;
         int recursive = 0;
-        int i;
-        for(i=1; zLeft[i]; i++){
-          if( zLeft[i]=='r' ) recursive = 1;
-          if( zLeft[i]=='d' ) detail = 1;
+        int j;
+        for(j=1; zLeft[j]; j++){
+          if( zLeft[j]=='r' ) recursive = 1;
+          if( zLeft[j]=='d' ) detail = 1;
         }
-        decode_trunk_page(iStart, pagesize, detail, recursive);
+        decode_trunk_page(iStart, detail, recursive);
         continue;
       }else{
         iEnd = iStart;
       }
-      if( iStart<1 || iEnd<iStart || iEnd>mxPage ){
+      if( iStart<1 || iEnd<iStart || iEnd>g.mxPage ){
         fprintf(stderr,
           "Page argument should be LOWER?..UPPER?.  Range 1 to %d\n",
-          mxPage);
+          g.mxPage);
         exit(1);
       }
       while( iStart<=iEnd ){
@@ -1056,6 +1258,6 @@ int main(int argc, char **argv){
       }
     }
   }
-  close(db);
+  fileClose();
   return 0;
 }
